package mmtext;

import java.awt.Color;
import java.awt.Point;
import java.awt.Window;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import javax.imageio.ImageIO;

import mmtext.datastruct.Stack;
import mmtext.images.BinaryImage;
import mmtext.images.ClassifierC45;
import mmtext.images.GrayScaleImage;
import mmtext.images.IBinaryImage;
import mmtext.images.IGrayScaleImage;
import mmtext.images.IImage;
import mmtext.images.IRGBImage;
import mmtext.images.MinimalBoundBox;
import mmtext.images.RGBImage;
import mmtext.images.descriptor.Moment;
import mmtext.images.gui.WindowImages;
import mmtext.images.gui.WindowXProjection;
import mmtext.images.operators.MorphologicalOperator;
import mmtext.images.operators.ThresholdGlobal;
import mmtext.images.operators.ThresholdLocal;
import mmtext.images.operators.Transformation;
import mmtext.images.operators.uao.MaxtreeBuilder;
import mmtext.images.operators.uao.UltimateOpening;
import mmtext.images.utils.ImageGraphics;
import mmtext.images.utils.ImageUtils;
import mmtext.images.utils.Pixel;
import mmtext.images.utils.Resampling;
import mmtext.images.utils.VanishingPoint;

public class MMText<NaiveUO> {

	private IRGBImage imgResult;
	private IGrayScaleImage img;
	private float resolution;
	private IGrayScaleImage imgTextRegionCandidate;
	private StringBuilder xml;
	private static IBinaryImage imgBinary;


	public MMText(IRGBImage image, float resolution){
		this.imgResult = Resampling.bilinear(image, resolution);
		this.img = imgResult.convertGrayScaleImage();
		this.resolution = resolution;
		xml = new StringBuilder("<taggedRectangles>\n");
	}

	public IGrayScaleImage getImgTextRegionCandidate(){
		return imgTextRegionCandidate;
	}

	public StringBuilder getXML(){
		xml.append("</taggedRectangles>\n");
		return xml;
	}

	/**
	 * Pega a imagem de entrada em tons de cinza
	 * @return
	 */
	public IGrayScaleImage getInputImage(){
		return img;
	}

	/**
	 * Pega o resultado
	 * @return
	 */
	public IRGBImage getResultImage(){
		return imgResult;
	}


	/**
	 * Gera um lista de regioes candidatas
	 * @param imgBinary
	 * @param anglos
	 * @param seResolution
	 * @return
	 */
	public ArrayList<IBinaryImage> createRegionCandidate(IBinaryImage imgBinary, int anglos[], int seResolution[]){
		ArrayList<IBinaryImage> imgs = new ArrayList<IBinaryImage>();
		for(int i=0; i < seResolution.length; i++){
			IBinaryImage se = MorphologicalOperator.createImageDecorator(255, 0, 1, seResolution[i]);
			for(int z=0; z < anglos.length; z++){
				IBinaryImage se2 = Resampling.rotationComplete(se, anglos[z]);
				se2.setOrigin(se2.getWidth()/2, se2.getHeight()/2);
				se2.setBackground(se.getBackground());
				IBinaryImage imgRegion = MorphologicalOperator.dilation(imgBinary, se2);

				imgs.add(imgRegion);
			}
		}
		return imgs;
	}

	/**
	 * Extrair as compoentes conexas da imagem
	 * @param args
	 * @throws IOException 
	 */
	public IBinaryImage extractionCC(int opcao) throws IOException {

		int indexT;  
		IBinaryImage imgUnion = null;
		IBinaryImage imgRb = null;
		IBinaryImage imgRb2 = null;
		if(opcao != 2){
			MaxtreeBuilder max = new MaxtreeBuilder();
			UltimateOpening uo = new UltimateOpening();
			uo.setGradualTransitions(0);

			max.buildMaxtree(img.invert(), 1);

			uo.runUO(max, img.getHeight()/3);

			IGrayScaleImage img2 = uo.getTransformImg();

			indexT = new ThresholdGlobal(img2).getIndexOtsu();
			indexT = (indexT > 10? 10: indexT);
			imgRb = new ThresholdLocal(img2).otsuLocal(indexT);
			imgRb = MorphologicalOperator.areaOpen(imgRb, 30);

			imgUnion = imgRb;
		}
		if(opcao != 1){
			MaxtreeBuilder max2 = new MaxtreeBuilder();
			UltimateOpening uo2 = new UltimateOpening();
			uo2.setGradualTransitions(0);
			max2.buildMaxtree(img, 1);
			uo2.runUO(max2, img.getHeight()/3);

			IGrayScaleImage img2 = uo2.getTransformImg();

			indexT = new ThresholdGlobal(img2).getIndexOtsu();
			indexT = (indexT > 10? 10: indexT);
			imgRb2 = new ThresholdLocal(img2).otsuLocal(indexT);
			imgRb2 = MorphologicalOperator.areaOpen(imgRb2, 30);
			imgUnion = imgRb2;
		}
		if(opcao == 0){
			imgUnion = MorphologicalOperator.union(imgRb, imgRb2);
		}
		return imgUnion;
	}

	public static MinimalBoundBox getCandidate(IBinaryImage img, int label){
		MinimalBoundBox box = new MinimalBoundBox(img, true, label);
		if(box.getMaxHeightPoint() == null || box.getMaxWidthPoint() == null || box.getMinHeightPoint() == null || box.getMinWidthPoint() == null)
			return  null;

		int area = box.getImage().getArea(); 
		double width = box.getWidth();
		double height = box.getHeight();
		double denc = area / (width * height); 
		boolean den = (denc >= 0.25);

		double hate = (Math.max(width, height) / Math.min(width, height));

		if (!(den)){
			return null;
		}
		else if (hate < 1.5){
			return null;
		}        

		return box;
	}

	public static IBinaryImage getConnectComponent(IGrayScaleImage img, int label, int back){
		IBinaryImage imgB = new BinaryImage(img.getWidth(), img.getHeight());
		imgB.setBackground(back);
		imgB.setId(label);
		int fore = (back == 0)? 255: 0;
		int count = 0;
		for(int i=0; i < imgB.getSize(); i++){
			if(img.getPixel(i) == label){
				imgB.setPixel(i, fore);
				count++; 
			}
			else
				imgB.setPixel(i, back);
		}
		if(count == 0) return null;
		return imgB;


	}

	public static void markArea8ConnectedByStack(IGrayScaleImage img, IBinaryImage imgB, Point seed, int newLabel){
		Stack<Point> q = new Stack<Point>();
		q.push(seed);
		while (!q.isEmpty()) {
			Point p = q.pop();
			if (ImageUtils.isIndexInImage(img, p.x, p.y) && imgB.getPixel(p.x, p.y) == imgB.getForeground() && img.getPixel(p.x, p.y) != newLabel ){
				img.setPixel(p.x,p.y, newLabel);
				q.push(new Point(p.x-1,p.y-1));
				q.push(new Point(p.x,p.y-1));
				q.push(new Point(p.x-1,p.y));
				q.push(new Point(p.x+1,p.y+1));
				q.push(new Point(p.x,p.y+1));
				q.push(new Point(p.x+1,p.y));
				q.push(new Point(p.x+1,p.y-1));
				q.push(new Point(p.x-1,p.y+1));
			}
		}
	}

	public ArrayList<MinimalBoundBox> getTextRecionCandidate(IBinaryImage imgRegion, IBinaryImage imgBinary, boolean flagMerge) throws Exception{
		imgTextRegionCandidate = MorphologicalOperator.getLabeling8ConnectedByStackGrayScale(imgRegion);
		ArrayList<MinimalBoundBox> listCandidate = new ArrayList<MinimalBoundBox>();
		for(int label = 1; label <= imgTextRegionCandidate.getPixelMax(); label++){
			IBinaryImage b = getConnectComponent(imgTextRegionCandidate, label, imgRegion.getBackground()); 
			if(b == null) continue;
			MinimalBoundBox box = getCandidate(b, label); 
			if(box == null){
				markArea8ConnectedByStack(imgTextRegionCandidate, imgRegion, Moment.getCentroid(b), 255);
			}else{
				listCandidate.add(box);
			}            
		}

		IBinaryImage imgB = imgRegion.duplicate();
		if(flagMerge){
			ImageGraphics.lineWidth = 2;
			int newLabel = imgTextRegionCandidate.getPixelMax()+1;
			int labelMaxOld = imgTextRegionCandidate.getPixelMax();
			int idMergeMerge;
			ArrayList<MinimalBoundBox> listCandidatesRemove = new ArrayList<MinimalBoundBox>();
			ArrayList<MinimalBoundBox> listNewCandidates = new ArrayList<MinimalBoundBox>();
			for(int k=0; k < listCandidate.size()-1; k++){
				MinimalBoundBox boxK = listCandidate.get(k);
				for(int j=k+1; j < listCandidate.size(); j++){
					MinimalBoundBox boxJ = listCandidate.get(j);
					idMergeMerge = -1;
					if(boxK.isMergeByCriterio2(boxJ) && boxK.isMergeByCriterio1(boxJ)){

						imgB = ImageGraphics.drawLine(imgB.convertGrayScale(), Moment.getCentroid(boxK.getImage()), Moment.getCentroid(boxJ.getImage()), Color.BLACK).convertGrayScaleImage().convertBinaryImage(100, imgB.getBackground());
						Point p = Moment.getCentroid(boxK.getImage());
						if(imgTextRegionCandidate.getPixel(p.x, p.y) > labelMaxOld){
							idMergeMerge = imgTextRegionCandidate.getPixel(p.x, p.y);
						}
						markArea8ConnectedByStack(imgTextRegionCandidate, imgB,  p, newLabel);
						listCandidatesRemove.add(boxK);
						listCandidatesRemove.add(boxJ); 

						IBinaryImage b = getConnectComponent(imgTextRegionCandidate, newLabel, imgB.getBackground());
						if(b == null) continue;
						if(idMergeMerge != -1){
							int i = -1;
							for(i=0; i < listNewCandidates.size(); i++){
								try{
									if(listNewCandidates.get(i).getImage().getId() == idMergeMerge){
										listNewCandidates.set(i, getCandidate(b, newLabel));
										break;
									}
								}catch(Exception e){

								}
							}
						}else{
							listNewCandidates.add(getCandidate(b, newLabel));
						}
						newLabel++;   

					}
				}
			}
			listCandidate.removeAll(listCandidatesRemove);
			listCandidate.addAll(listNewCandidates);

		}

		return listCandidate;
	}


	public void run(IBinaryImage imgBinary, ArrayList<MinimalBoundBox> listCandidate) throws Exception{ 
		ArrayList<MinimalBoundBox> listCandidatesRemove = new ArrayList<MinimalBoundBox>();
		IRGBImage imgResult2 = imgResult.duplicate();
		for(int i=0; i < listCandidate.size(); i++){
			MinimalBoundBox box = listCandidate.get(i);
			if(box == null) continue;
			IImage imgsAffine[] = affineTransform(img, box.getImage(imgBinary));
			IGrayScaleImage imgGr = (IGrayScaleImage) imgsAffine[0];
			IBinaryImage imgBr = (IBinaryImage) imgsAffine[1];
			if(box == null || !classify(imgGr, imgBr, box)){
				listCandidatesRemove.add(box);
			}else{
				ImageGraphics.lineWidth = 2;
				IBinaryImage imgBr2 = imgBr;
				WindowXProjection projection1 = new WindowXProjection(imgBr2);
				IBinaryImage se = MorphologicalOperator.createImageDecorator(255, 0, 2 * projection1.getCharacterSpaceWidth(),1 );
				imgBr = MorphologicalOperator.dilationBoundBox(imgBinary, se, box.getMinWidthPoint().x, box.getMinHeightPoint().y, box.getMaxWidthPoint().x, box.getMaxHeightPoint().y);
				imgTextRegionCandidate = MorphologicalOperator.getLabeling8ConnectedByStackGrayScale(imgBr);

				for(int label = 1; label <= imgTextRegionCandidate.getPixelMax(); label++){
					IBinaryImage b = getConnectComponent(imgTextRegionCandidate, label, imgBr.getBackground()); 
					if(b == null) continue;

					MinimalBoundBox box1 = new MinimalBoundBox(box, b);

					if(box1 != null){
						box1.paint(imgResult2, Color.GREEN, this.resolution);
						xml.append(box1.getXML());
					}            
				}

				ImageGraphics.lineWidth = 1;                
			}
		}
		imgResult = imgResult2;
		ImageGraphics.lineWidth = 2;
	}

	public IImage[] affineTransform(IGrayScaleImage imgG, IBinaryImage imgBin){

		int range = 20;

		IBinaryImage imgB2 = imgBin;
		int topCount = -1; //valor do histograma para a melhor topLine
		int topPointX = -1;
		int topPointY = -1;
		float topRot = -1;
		int baseCount = -1;
		int basePointX = -1;
		int basePointY = -1;
		float baseRot = -1;
		int partSum;
		int maxSum = -1;
		int angleSum = 0;
		int hist[];
		int i;

		for(float f = -range; f<=range;f+=(float)range/40){
			IBinaryImage imgB3 = Resampling.rotationBinary(imgB2, 360+f);
			hist = imgB3.getTopYProjection();

			for(i = 0; i < imgB3.getHeight();i++){	
				if(hist[i] > topCount){
					topCount = hist[i];
					topPointY = i;
					topRot = f;
					if(f < 0)
						for(int j=imgB3.getWidth()- 1;;j--){
							if(!imgB3.isPixelBackground(j, i)){
								topPointX = j;
								break;
							}
						}
					else{
						for(int j=0;;j++){
							if(!imgB3.isPixelBackground(j, i)){
								topPointX = j;
								break;
							}
						}
					}

				}
			}	
			hist = imgB3.getBaseYProjection();

			for(i = 0; i < imgB3.getHeight();i++){	
				if(hist[i] > baseCount){
					baseCount = hist[i];
					basePointY = i;
					baseRot = f;
					if(f < 0)
						for(int j=0;;j++){
							if(!imgB3.isPixelBackground(j, i)){
								basePointX = j;
								break;
							}
						}
					else{
						for(int j=imgB3.getWidth()- 1;;j--){
							if(!imgB3.isPixelBackground(j, i)){
								basePointX = j;
								break;
							}
						}
					}

				}
			}	
		}

		IGrayScaleImage imgG2 = Resampling.rotation(imgG, baseRot);

		for(int angle = -45;angle<=45;angle+=3){
			IBinaryImage imgB3 = Resampling.shearBinary(imgB2, angle);
			imgB3 = ImageUtils.adjustImage(imgB3);
			hist = imgB3.getHistogramXprojection();
			partSum = 0;
			for(i = 0; i < imgB3.getWidth();i++){	
				partSum += hist[i]*hist[i]; 
			}
			if(partSum > maxSum){
				maxSum = partSum;
				angleSum = angle;
			}
		}
		Pixel aux = new Pixel(-1,-1);
		Pixel topPixel = aux.rotatePixel(new Pixel(topPointX,topPointY), topRot, imgB2.getWidth()/2, imgB2.getHeight()/2);
		Pixel basePixel = aux.rotatePixel(new Pixel(basePointX,basePointY), baseRot, imgB2.getWidth()/2, imgB2.getHeight()/2);
		Pixel p = VanishingPoint.calcVanishingPoint(topPixel.getX(),topPixel.getY(),topRot,basePixel.getX(),basePixel.getY(),baseRot);

		IImage imgHomography[] = Transformation.homography(imgBin, imgG, p.getX(), p.getY(), angleSum);
		imgB2 = (IBinaryImage) imgHomography[0];
		imgG2 = (IGrayScaleImage) imgHomography[1];

		int bounds[] = ImageUtils.imageBounds(imgB2);
		int beginWidth = bounds[0];
		int beginHeight = bounds[1];
		int endWidth = bounds[2];
		int endHeight = bounds[3];
		if(endWidth < beginWidth){
			imgB2 = imgBin;
			imgG2 = imgG;
			bounds = ImageUtils.imageBounds(imgBin);
			beginWidth = bounds[0];
			beginHeight = bounds[1];
			endWidth = bounds[2];
			endHeight = bounds[3];

		}

		IBinaryImage imgB3 = new BinaryImage(endWidth - beginWidth + 1, endHeight - beginHeight+1); //o tamanho da imagem sao os limites + 10 //10 e uma folga
		IGrayScaleImage imgG3 = new GrayScaleImage(endWidth - beginWidth + 1, endHeight - beginHeight+1); //o tamanho da imagem sao os limites + 10 //10 e uma folga
		imgB3.setBackground(imgB2.getBackground());
		for(int w= beginWidth, ii = 0; w < endWidth+1; w++, ii++){ //w para imagem original e i para nova imagem 
			for(int h=beginHeight, j =0; h < endHeight+1; h++, j++){ //h para imagem original e j para nova imagem

				if(ImageUtils.isIndexInImage(imgB3, ii, j)){
					imgB3.setPixel(ii, j, imgB2.getPixel(w, h));
					imgG3.setPixel(ii, j, imgG2.getPixel(w, h));
				}
				else
					imgB3.setPixel(ii, j, imgB2.getBackground());
			}
		}


		return new IImage[]{imgG3, imgB3};
	}

	public boolean classify(IGrayScaleImage imgGRot, IBinaryImage imgBr, MinimalBoundBox box){
		try{
			WindowXProjection projection1 = new WindowXProjection(imgBr);
			//atributos da projecao paper do Wu
			double charHVariance = projection1.getCharacterHeightVariance();
			double charWVariance = projection1.getCharacterWidthVariance(); 
			double linearity = projection1.getLinearity(); 
			double countChar = projection1.getCountCharacters();

			double homogenity = homogenity(imgGRot, imgBr);

			double result = ClassifierC45.classify(new Double[]{charHVariance, charWVariance, linearity, homogenity, countChar});
			if(result == 1)
				return true; 
			else	
				return false;            
		}catch(Exception e){
			return false;
		}
	}

	public static double homogenity(IGrayScaleImage imgG, IBinaryImage imgB){
		/////calculando a homogeneidade
		int w = imgB.getWidth();
		int h = imgB.getHeight();
		double sum = 0;
		int count = 0;
		for(int ii=0; ii < w; ii++){
			for(int jj=0; jj < h; jj++){
				if(!imgB.isPixelBackground(ii, jj)){
					sum += imgG.getPixel(ii, jj);
					count++;
				}
			}
		}

		double media = sum / (double) count;
		double var = 0;
		for(int ii=0; ii < w; ii++){
			for(int jj=0; jj < h; jj++){
				if(!imgB.isPixelBackground(ii, jj)){
					var += Math.pow(imgG.getPixel(ii, jj) - media,2);
				}
			}
		}
		var = Math.sqrt(var / count);

		return var;
	}


	/**
	 * @param args
	 * @throws Exception 
	 */
	public static void main(String[] args) throws Exception {

		boolean display = false;

		/***************Parametros de entrada do metodo *******************/
		int anglos[] = new int[]{90};//0, 45, 90, 135};
		int seResolution[] = new int[]{15};
		float resolution = 0.5f;
		String path = "img/Peacocks.JPG";
		/******************************************************************/

		IRGBImage imgColor = new RGBImage(ImageIO.read(new File(path)));

		if(display) WindowImages.show(imgColor, "Imagem de entrada");
		MMText mmtext = new MMText(imgColor, resolution);

		if(display) WindowImages.show(mmtext.getInputImage(), "Imagem de entrada em tons de cinza");

		IBinaryImage imgBinary = mmtext.extractionCC(0);
		if(display) WindowImages.show(MorphologicalOperator.getLabeling8ConnectedByStackColor(imgBinary), "Extracao de componentes conexas");

		ArrayList<IBinaryImage> listRegionCandidate = mmtext.createRegionCandidate(imgBinary, anglos, seResolution);
		if(display) WindowImages.show(MorphologicalOperator.getLabeling8ConnectedByStackColor(listRegionCandidate.get(0)), "Geracao de Region candidate");

		for(IBinaryImage imgRegion: listRegionCandidate){
			ArrayList<MinimalBoundBox> listTextRegionCandidate = mmtext.getTextRecionCandidate(imgRegion, imgBinary, true);
			if(display) WindowImages.show(MorphologicalOperator.getLabeling8ConnectedByStackColor(ThresholdGlobal.manual(mmtext.getImgTextRegionCandidate(), 1)), "Text region candidate");
			mmtext.run(imgBinary, listTextRegionCandidate);
			long tempoFinal = System.currentTimeMillis();
			WindowImages.show(mmtext.getResultImage(), "Classification of text region");
		}
	}
}