package mmtext.datastruct;

import java.util.ArrayList;

public class PriorityQueue<T> {

    private int numElements = 0;

    private int priorityLowerBound = 0;

    private ArrayList<Queue<T>> queues;

    private int maxPriority;

    /**
     * Creates an priority queue to store elements with priorities up to
     * <code>maxPriority</code>.
     * 
     * @param maxPriority
     *            the maximum priority given to an element in this queue
     */
    public PriorityQueue(int maxPriority) {
        this.maxPriority = maxPriority;
        queues = new ArrayList<Queue<T>>();
        for (int i = 0; i <= maxPriority; i++) {
            queues.add(new Queue<T>());
        }
    }

    /**
     * Returns the element with lowest priority.
     * 
     * @return the element with lowest priority, <code>null</code> if this
     *         queue is empty
     */
    public T remove() {
        for (int i = priorityLowerBound; i <= maxPriority; i++) {
            if (!queues.get(i).isEmpty()) {
                numElements--;
                priorityLowerBound = i;
                return queues.get(i).dequeue();
            }
        }
        return null;
    }

    

    
    /**
     * Appends the specified element to the queue with the given priority.
     * 
     * @param element
     *            element to be appended to this queue
     * @param priority
     *            priority given to the element to be appended
     */
    public void add(T element, int priority) {
        if (priority < priorityLowerBound) {
            priorityLowerBound = priority;
        }
        numElements++;
        queues.get(priority).enqueue(element);
    }


    /**
     * Returns <code>true</code> if this queue contains no elements.
     * 
     * @return <code>true</code> if this queue contains no elements
     */
    public boolean isEmpty() {
        return (numElements == 0);
    }

    /**
     * Returns the number of elements in this queue.
     * 
     * @return the number of elements in this queue
     */
    public int size() {
        return numElements;
    }

}
