package mmtext.images.gui;

import java.awt.Color;
import java.awt.Font;
import java.awt.Point;
import java.text.DecimalFormat;
import java.text.NumberFormat;

import mmtext.images.IGrayScaleImage;
import mmtext.images.IRGBImage;
import mmtext.images.RGBImage;
import mmtext.images.operators.Histogram;
import mmtext.images.utils.ImageGraphics;

/**
 * 
 * @author wonder
 * Classe utilizado para exibir o histograma da imagem na tela
 */
public class WindowHistogram {
    static final int WIN_WIDTH = 300;
    static final int WIN_HEIGHT = 210;
    static final int HIST_WIDTH = 256;
    static final int HIST_HEIGHT = 128;
    static final int BAR_HEIGHT = 12;
    static final int XMARGIN = 20;
    static final int YMARGIN = 10;
    IRGBImage imgOut;
    /**
     * Desenha o histograma
     * @param img
     * @return
     */
    public void drawHistogram(IGrayScaleImage img){
    	imgOut = new RGBImage(WIN_WIDTH, WIN_HEIGHT);
        imgOut.initImage(255);
        Histogram histogram = new Histogram(img);
        int h[] = histogram.getHistogram(); 
        
        int maxCount = 0;

        for (int i = 0; i<h.length; i++){
            if (h[i] > maxCount) {
                maxCount = h[i];
            }
        }
        
        drawPlot(maxCount, h);
        int x = XMARGIN + 1;
        int y = YMARGIN + HIST_HEIGHT + 2;
        drawUnscaledColorBar(x-1, y, 256, BAR_HEIGHT);
        y += BAR_HEIGHT+2;
        drawText(histogram, x, y);
    }
    
    public IRGBImage getGraphics(){
    	return imgOut;
    }
    
    /**
     * Desenha o texto no histograma
     * @param imgOut
     * @param histogram
     * @param x
     * @param y
     */
    private void drawText(Histogram histogram, int x, int y) {
        NumberFormat formatter = new DecimalFormat("0.00");
        
        ImageGraphics.drawString(imgOut, "0", new Font(null, Font.PLAIN, 10), Color.BLACK, x, y);
        ImageGraphics.drawString(imgOut, "255", new Font(null, Font.PLAIN, 10), Color.BLACK, x+241, y);
        y += 20;
        ImageGraphics.drawString(imgOut, "Mean: " + formatter.format(histogram.getMean()), new Font(null, Font.PLAIN, 10), Color.BLACK, x, y);
        ImageGraphics.drawString(imgOut, "Pixel Min: " + histogram.getPixelMin(), new Font(null, Font.PLAIN, 10), Color.BLACK, x + 160, y);
        y+=12;
        ImageGraphics.drawString(imgOut, "Standard Deviatio: " + formatter.format(histogram.getStandardDeviatio()), new Font(null, Font.PLAIN, 10), Color.BLACK, x, y);
        
        ImageGraphics.drawString(imgOut, "Pixel Max: " + histogram.getPixelMax(), new Font(null, Font.PLAIN, 10), Color.BLACK, x + 160, y);
        y+=12;
        ImageGraphics.drawString(imgOut, "Peak Max: " + histogram.getMaxPeak(), new Font(null, Font.PLAIN, 10), Color.BLACK, x, y);
        ImageGraphics.drawString(imgOut, "Peak Mean: " + formatter.format(histogram.getMeanPeak()), new Font(null, Font.PLAIN, 10), Color.BLACK, x + 160, y);
    
    }
    
    /**
     * Desenhar os picos do histograma
     * @param img
     * @param maxCount
     * @param h
     */
    private void drawPlot(int maxCount, int h[]) {
        if (maxCount==0) maxCount = 1;
       
        ImageGraphics.drawRectangle(imgOut, new Point(XMARGIN, YMARGIN), HIST_WIDTH+2, HIST_HEIGHT+1, Color.BLACK);
        int  y;
        for (int i = 0; i<HIST_WIDTH; i++) {
            y = (int)((double)HIST_HEIGHT*h[i])/maxCount;
            if (y>HIST_HEIGHT)
                y = HIST_HEIGHT;
            ImageGraphics.drawLine(imgOut, new Point(i+XMARGIN, YMARGIN+HIST_HEIGHT), new Point(i+XMARGIN, YMARGIN+HIST_HEIGHT-y), Color.BLACK);
        }
    }
    
    public void drawLineVerticle(int x){
    	ImageGraphics.drawLine(imgOut, new Point(x+XMARGIN, YMARGIN), new Point(x+XMARGIN, YMARGIN+HIST_HEIGHT), Color.RED);
    }
    
    /**
     * Desenha a barra de escala de niveis de cinza
     * @param img
     * @param x
     * @param y
     * @param width
     * @param height
     */
    private void drawUnscaledColorBar(int x, int y, int width, int height) {
        for (int i = 0; i < 256; i++) {
           ImageGraphics.drawLine(imgOut, new Point(x + i, y + 0), new Point(x + i, y + height), new Color(i,i,i));
        }
        ImageGraphics.drawRectangle(imgOut, new Point(x, y), width+2, height, Color.BLACK);
    }
    
    
    public static void show(IGrayScaleImage img){
        WindowHistogram win = new WindowHistogram();
        win.drawHistogram(img);
        WindowImages.show(win.getGraphics(), "Histogram");
    }
    

}
