package mmtext.images.operators;

import java.util.ArrayList;

import mmtext.datastruct.Stack;
import mmtext.images.BinaryImage;
import mmtext.images.GrayScaleImage;
import mmtext.images.IBinaryImage;
import mmtext.images.IGrayScaleImage;
import mmtext.images.IRGBImage;
import mmtext.images.RGBImage;
import mmtext.images.utils.ImageUtils;
import mmtext.images.utils.Pixel;

public class MorphologicalOperator {



    /**
     * Erosao em niveis de cinza pelo elemento estruturante flat
     * @param img - imagem de entrada
     * @param se - elemento estruturante flat
     * @return imagem erodita
     */
    public static IGrayScaleImage erosion(IGrayScaleImage img, IBinaryImage se){
        int min;
        IGrayScaleImage imgM = new GrayScaleImage(img.getWidth(), img.getHeight());
        for(int w=0; w < img.getWidth(); w++){
            for(int h=0; h < img.getHeight(); h++){
                min = Integer.MAX_VALUE; 
                for(int x=-se.getWidth()/2, x1=0; x1 < se.getWidth(); x++, x1++){ //Varendo o elemento estruturante
                    for(int y=-se.getHeight()/2, y1=0; y1 < se.getHeight(); y++, y1++){
                        if(se.getPixel(x1, y1) != se.getForeground()){
                            continue;
                        }else if(!ImageUtils.isIndexInImage(img,w + x, h + y))
                            continue;
                        else if(min > img.getPixel(w + x, h + y) )//&& se.getForeground() == BinaryImage.COLOR_BINARY_IMAGE_BLACK)
                            min = img.getPixel(w + x, h + y);
                    }
                }
               // if( min != Integer.MAX_VALUE)
                    imgM.setPixel(w, h, min);
            }
        }
        return imgM;
    }
   
    /**
     * Dilatacao em niveis de cinza pelo elemento estruturante flat
     * @param img - imagem de entrada
     * @param se - elemento estruturante flat
     * @return imagem dilatada
     */
    public static IGrayScaleImage dilation(IGrayScaleImage img, IBinaryImage se){
        int max;
        IGrayScaleImage imgM = new GrayScaleImage(img.getWidth(), img.getHeight());
        for(int w=0; w < img.getWidth(); w++){
            for(int h=0; h < img.getHeight(); h++){
                max = Integer.MIN_VALUE;
                for(int x=-se.getWidth()/2, x1=0; x1 < se.getWidth(); x++, x1++){ //Varendo o elemento estruturante
                    for(int y=-se.getHeight()/2, y1=0; y1 < se.getHeight(); y++, y1++){
                        if(se.getPixel(x1, y1) != se.getForeground()){
                            continue;
                        }else if(!ImageUtils.isIndexInImage(img,w + x, h + y))
                            continue;
                        else if(max < img.getPixel(w + x, h + y) )//&& se.getForeground() == BinaryImage.COLOR_BINARY_IMAGE_BLACK)
                            max = img.getPixel(w + x, h + y);
                    }
                }
                //if( max != Integer.MIN_VALUE)
                    imgM.setPixel(w, h, max);
            }
        }
        return imgM;
    }
    
    /**
     * Dilatacao para imagens binarias
     * @param img
     * @param seOrinal
     * @return
     */
    public static IBinaryImage dilation(IBinaryImage img, IBinaryImage seOrinal){
        int foreGround = img.getForeground();
        IBinaryImage imgM = new BinaryImage(img.getWidth(), img.getHeight());        
        imgM.setBackground(img.getBackground());
        imgM.initImage(img.getBackground()); 
          
        IBinaryImage se = seOrinal.reflection(seOrinal);
        se.setOrigin(seOrinal.getOriginX(), seOrinal.getOriginY());
        int xOrigem = se.getOriginX() - se.getWidth()/2;
        int yOrigem = se.getOriginY() - se.getHeight()/2;
        
        for(int w=0; w < img.getWidth() + se.getWidth(); w++){
            for(int h=0; h < img.getHeight() + se.getHeight(); h++){
                
                for(int xx=0, x=-se.getWidth()/2; xx < se.getWidth(); x++,xx++){ //Varendo o elemento estruturante
                    for(int yy=0,y=-se.getHeight()/2; yy < se.getHeight(); yy++, y++){
                       
                        if(ImageUtils.isIndexInImage(img, w + x, h + y)){ //checando as retricoes da img
                            if((img.getPixel(w + x, h + y) == foreGround || se.getPixel(xx, yy) == BinaryImage.DONT_CARE) && se.getPixel(xx, yy) == foreGround){ //verificando se o SE esta dentro da imagem
                                if(ImageUtils.isIndexInImage(imgM, w + xOrigem, h + yOrigem)){    
                                    imgM.setPixel(w + xOrigem, h + yOrigem, foreGround);
                                }
                            }
                        }
                    }
                }
                
            }
        }
        return  imgM;
    } 
    
    
    public static IBinaryImage dilationBoundBox(IBinaryImage img, IBinaryImage seOrinal, int w1, int h1, int w2, int h2){
        int foreGround = img.getForeground();
        IBinaryImage imgM = new BinaryImage(img.getWidth(), img.getHeight());        
        imgM.setBackground(img.getBackground());
        imgM.initImage(img.getBackground());
        
        IBinaryImage se = seOrinal.reflection(seOrinal);
        se.setOrigin(seOrinal.getOriginX(), seOrinal.getOriginY());
        int xOrigem = se.getOriginX() - se.getWidth()/2;
        int yOrigem = se.getOriginY() - se.getHeight()/2;
        
        for(int w=0; w < img.getWidth() + se.getWidth(); w++){
            for(int h=0; h < img.getHeight() + se.getHeight(); h++){
                
                for(int xx=0, x=-se.getWidth()/2; xx < se.getWidth(); x++,xx++){ //Varendo o elemento estruturante
                    for(int yy=0,y=-se.getHeight()/2; yy < se.getHeight(); yy++, y++){
                       
                        if(ImageUtils.isIndexInImage(img, w + x, h + y) && w + x >= w1 && w + x <= w2 && h + y >= h1 && h + y <= h2){ //checando as retricoes da img
                            if((img.getPixel(w + x, h + y) == foreGround || se.getPixel(xx, yy) == BinaryImage.DONT_CARE) && se.getPixel(xx, yy) == foreGround){ //verificando se o SE esta dentro da imagem
                                if(ImageUtils.isIndexInImage(imgM, w + xOrigem, h + yOrigem)){    
                                    imgM.setPixel(w + xOrigem, h + yOrigem, foreGround);
                                }
                            }
                        }
                    }
                }
                
            }
        }
        return  imgM;
    } 
   
    
    /**
     * Rotular os componentes 8-conectados utilizando uma pilha
     * @param img
     * @return
     */
    public static IGrayScaleImage getLabeling8ConnectedByStackGrayScale(IBinaryImage imgIn){
    	IBinaryImage img = imgIn.duplicate();
        IGrayScaleImage imgOut = new GrayScaleImage(img.getWidth(), img.getHeight());
        
        int labeling = 0;
        for(int w=0; w < img.getWidth(); w++)
            for(int h=0; h < img.getHeight(); h++){
            	//System.out.println("a = "+ img.getPixel(w, h));
                if(img.getPixel(w, h) == img.getForeground()){
                	labeling++;
                    Stack<Pixel> q = new Stack<Pixel>();
                    q.push(new Pixel(w,h));
                    while (!q.isEmpty()) {
                    	Pixel p = q.pop();
                        if (ImageUtils.isIndexInImage(img, p.getX(), p.getY()) && img.getPixel(p.getX(), p.getY()) == img.getForeground()) {

                            imgOut.setPixel(p.getX(),p.getY(), labeling);
                            img.setPixel(p.getX(),p.getY(), BinaryImage.DONT_CARE);
                            
                            q.push(new Pixel(p.getX()-1,p.getY()-1));
                            q.push(new Pixel(p.getX(),p.getY()-1));

                            q.push(new Pixel(p.getX()-1,p.getY()));
                            q.push(new Pixel(p.getX()+1,p.getY()+1));
                            q.push(new Pixel(p.getX(),p.getY()+1));
                            q.push(new Pixel(p.getX()+1,p.getY()));
                            q.push(new Pixel(p.getX()+1,p.getY()-1));
                            q.push(new Pixel(p.getX()-1,p.getY()+1));
                        }
                    }
                }
            }
        
        return imgOut;
    }

    /**
     * Rotular os componentes 8-conectados utilizando uma pilha
     * @param img
     * @return
     */
    public static IRGBImage getLabeling8ConnectedByStackColor(IBinaryImage imgIn){
    	IBinaryImage img = imgIn.duplicate();
        IRGBImage imgOut = new RGBImage(img.getWidth(), img.getHeight());
        
        int labeling = 0;
        for(int w=0; w < img.getWidth(); w++)
            for(int h=0; h < img.getHeight(); h++){
                if(img.getPixel(w, h) == img.getForeground()){
                	labeling++;
                    Stack<Pixel> q = new Stack<Pixel>();
                    q.push(new Pixel(w,h));
                    while (!q.isEmpty()) {
                    	Pixel p = q.pop();
                        if (ImageUtils.isIndexInImage(img, p.getX(), p.getY()) && img.getPixel(p.getX(), p.getY()) == img.getForeground()) {

                            imgOut.setPixel(p.getX(),p.getY(), ImageUtils.getColor(labeling, 1000));
                            img.setPixel(p.getX(),p.getY(), BinaryImage.DONT_CARE);
                            
                            q.push(new Pixel(p.getX()-1,p.getY()-1));
                            q.push(new Pixel(p.getX(),p.getY()-1));

                            q.push(new Pixel(p.getX()-1,p.getY()));
                            q.push(new Pixel(p.getX()+1,p.getY()+1));
                            q.push(new Pixel(p.getX(),p.getY()+1));
                            q.push(new Pixel(p.getX()+1,p.getY()));
                            q.push(new Pixel(p.getX()+1,p.getY()-1));
                            q.push(new Pixel(p.getX()-1,p.getY()+1));
                        }
                    }
                }
            }
        
        return imgOut;
    }

    
    /**
     *
     * @param img - imagem de entrada
     * @param area - tamanho minimo da area
     * @return - imagem com componentes conexas de area >= param_area
     */
    public static IBinaryImage areaOpen(IBinaryImage img, int area){
        IGrayScaleImage imgLabel = getLabeling8ConnectedByStackGrayScale(img);
        

        int max = imgLabel.getPixelMax();
        int hist[] = new int[max+1];
        for(int w=0; w < imgLabel.getWidth(); w++){
            for(int h=0; h < imgLabel.getHeight(); h++){
                hist[imgLabel.getPixel(w, h)]++;
            }
        }

        IBinaryImage imgOut = new BinaryImage(imgLabel.getWidth(), imgLabel.getHeight());
        imgOut.setBackground(img.getBackground());
        imgOut.initImage(img.getBackground());

        for(int w=0 ; w < imgOut.getWidth(); w++){
            for(int h=0; h < imgOut.getHeight(); h++){
                if(hist[imgLabel.getPixel(w,h)] >= area){
                    imgOut.setPixel(w, h, img.getPixel(w, h));
                }
            }
        }
        return imgOut;
    }
    

    /**
     * Retorna uma lista com as altura das CCs
     */
    public static ArrayList<Integer> getHeights(IBinaryImage imgIn){
    	IBinaryImage img = imgIn.duplicate();
    	ArrayList<Integer> list = new ArrayList<Integer>();
        IGrayScaleImage imgOut = new GrayScaleImage(img.getWidth(), img.getHeight());
        
        int labeling = 0;
        for(int w=0; w < img.getWidth(); w++)
            for(int h=0; h < img.getHeight(); h++){
            	//System.out.println("a = "+ img.getPixel(w, h));
                if(img.getPixel(w, h) == img.getForeground()){
                	labeling++;
                    Stack<Pixel> q = new Stack<Pixel>();
                    q.push(new Pixel(w,h));
                    int max = 0;
                    int min = img.getHeight();
                    while (!q.isEmpty()) {
                    	Pixel p = q.pop();
                        if (ImageUtils.isIndexInImage(img, p.getX(), p.getY()) && img.getPixel(p.getX(), p.getY()) == img.getForeground()) {
                        	
                            imgOut.setPixel(p.getX(),p.getY(), labeling);
                            img.setPixel(p.getX(),p.getY(), BinaryImage.DONT_CARE);
                            
                            q.push(new Pixel(p.getX()-1,p.getY()-1));
                            q.push(new Pixel(p.getX(),p.getY()-1));
                            q.push(new Pixel(p.getX()-1,p.getY()));
                            q.push(new Pixel(p.getX()+1,p.getY()+1));
                            q.push(new Pixel(p.getX(),p.getY()+1));
                            q.push(new Pixel(p.getX()+1,p.getY()));
                            q.push(new Pixel(p.getX()+1,p.getY()-1));
                            q.push(new Pixel(p.getX()-1,p.getY()+1));
                            
                            if(p.getY() > max){
                            	max  = p.getY();
                            }
                            if(p.getY() < min){
                            	min = p.getY();
                            }
                        }
                    }
                    list.add(max - min + 1);
                }
            }
        
        return list;
    	
    }
    
    
    /**
     * Uniao de duas imagens
     * @param img1 - BinaryImage
     * @param img2 - BinaryImage
     * @return BinaryImage com a uniao entre duas imagens
     */
    public static IBinaryImage union(IBinaryImage img1, IBinaryImage img2){
        IBinaryImage imgOut = new BinaryImage(img1.getWidth(), img1.getHeight());
        imgOut.setBackground(img1.getBackground());
        for(int i=0; i < img1.getWidth(); i++){
            for(int j=0; j < img1.getHeight(); j++){
            	imgOut.setPixel(i, j, (!img1.isPixelBackground(i, j) || !img2.isPixelBackground(i, j)? img1.getForeground() : img1.getBackground() ));
            }
        }
        return imgOut;
    }
    
    public static IBinaryImage createImageDecorator(int background, int foreground, int n, int m) {
        IBinaryImage se = new BinaryImage(n, m);
        se.setOrigin(n/2, m/2);
        se.setBackground(background);
        se.initImage(foreground);
        return se;
    }
}
