package mmtext.images.operators;

import mmtext.images.BinaryImage;
import mmtext.images.IBinaryImage;
import mmtext.images.IGrayScaleImage;


public class ThresholdGlobal {

    private IGrayScaleImage img;
    

    public ThresholdGlobal(IGrayScaleImage img ){
        this.img = img;        
    }
    
    public static IBinaryImage otsu(IGrayScaleImage img){
    	return new ThresholdGlobal(img).otsu();
    }
    
    
    public static IBinaryImage manual(IGrayScaleImage img, int limiar){

        IBinaryImage imgOut = new BinaryImage(img.getWidth(), img.getHeight());        
        
        for(int i = 0; i < imgOut.getWidth(); i++){
            for(int j = 0; j < imgOut.getHeight(); j++){
                if(img.getPixel(i, j) <= limiar){
                    imgOut.setPixel(i, j, 255); //0bj
                }
                else{
                	imgOut.setPixel(i, j, 0); //fundo
                }
            }
        }

        return imgOut;
    }

    public int getIndexOtsu(){
    	return this.argmax(img);
    }
    
    public IBinaryImage otsu(){

        IBinaryImage imgOut = new BinaryImage(img.getWidth(), img.getHeight());        
        int limiar = getIndexOtsu();

        for(int i = 0; i < imgOut.getWidth(); i++){
            for(int j = 0; j < imgOut.getHeight(); j++){
                if(img.getPixel(i, j) <= limiar){
                    imgOut.setPixel(i, j, 255); //0bj
                }
                else{
                	imgOut.setPixel(i, j, 0); //fundo
                }
            }
        }

        return imgOut;
    }

    private int argmax(IGrayScaleImage img){

        int argmax = 0;
        double valueMax = 0; 
        double value = 0;
        double omega1 = 0;
        double omega2 = 0;
        double m1 = 0, m2 = 0, ms = 0, miT = 0;
        double sigmaB = 0, sigmaT = 0;

        double[] relativeHistogram = this.getRelativeHistogram(img);
        
        for(int i = 0; i < relativeHistogram.length; i++){
            miT += i * relativeHistogram[i];
        }

        for(int i = 0; i < relativeHistogram.length; i++){
            sigmaT += Math.pow(i - miT, 2) * relativeHistogram[i];
        }       

        for(int t = 0; t < relativeHistogram.length; t++){

        	omega1 += relativeHistogram[t];
        	ms += t * relativeHistogram[t];
        	
            omega2 = 1 - omega1;
            m1 = ms / omega1;
            m2 = (miT - ms) / omega2;

            sigmaB = omega1 * omega2 * Math.pow(m1 - m2, 2);            

            value = Math.pow(sigmaB, 2)/Math.pow(sigmaT, 2);

            if(value > valueMax){
                valueMax = value;
                argmax = t;
            }
        }               
        if(argmax > 30)
        	argmax = 30;
        return argmax;
    }


   
    private double[] getRelativeHistogram(IGrayScaleImage img){

        int histogram[] = img.getHistogram();

        double relativeHistogram[] = new double[histogram.length];

        for(int i = 0; i < histogram.length; i++){
            relativeHistogram[i] = (double) histogram[i] / (img.getWidth() * img.getHeight());
        }        
        return relativeHistogram;
    }
}

