package mmtext.images.operators.uao;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.awt.image.ImageProducer;
import java.awt.image.MemoryImageSource;
import java.awt.image.PixelGrabber;
import java.awt.image.WritableRaster;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import mmtext.images.GrayScaleImage;
import mmtext.images.IBinaryImage;
import mmtext.images.IGrayScaleImage;

/**
 * 
 * @author alexandremorimitsu
 *
 *
 * @description
 * Classe manipuladora de imagens
 */
public class ImageManipulator {
	
	public static GrayScaleImage reader(String fileName) throws IOException {
		Image image = ImageIO.read(new File("img/" + fileName));
		BufferedImage bi = null;
		if (image instanceof BufferedImage)
			bi = (BufferedImage) image;
		else {
			int w = image.getWidth(null);
			int h = image.getHeight(null);
			bi = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
			Graphics2D g = bi.createGraphics();
			g.drawImage(image, 0, 0, null);
			g.dispose();
		}

		int width = bi.getWidth();
		int height = bi.getHeight();
		GrayScaleImage imgOut = new GrayScaleImage(width, height);
		int rgb, r, g, b;
		for (int w = 0; w < width; w++) {
			for (int h = 0; h < height; h++) {
				rgb = bi.getRGB(w, h);
				r = (int) ((rgb & 0x00FF0000) >>> 16); // Red level
				g = (int) ((rgb & 0x0000FF00) >>> 8); // Green level
				b = (int) (rgb & 0x000000FF); // Blue level
				imgOut.setPixel(w, h, (int) Math.round(.299 * r + .587 * g + .114 * b)); // convertendo para niveis de cinza
			}
		}
		return imgOut;
	}
	
    public static void showWindowImage(IGrayScaleImage imgIn, String title){
    	
    	int width = imgIn.getWidth();
    	int height = imgIn.getHeight(); 
        
        BufferedImage bi = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        WritableRaster raster = bi.getRaster();
        for(int w=0;w<width;w++){
            for(int h=0;h<height;h++){
                raster.setSample(w,h,0,imgIn.getPixel(w, h));
                raster.setSample(w,h,1,imgIn.getPixel(w, h));
                raster.setSample(w,h,2,imgIn.getPixel(w, h));
            }
        }
        ImageProducer producer = bi.getSource();
        int[] pix = new int[width * height];
        PixelGrabber grabber = new PixelGrabber(producer, 0, 0, width, height, pix, 0, width);
        try {
            if (!grabber.grabPixels())
                throw new IllegalStateException();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        MemoryImageSource membo = new MemoryImageSource(width,height,pix,0,width);
        Image img =  Toolkit.getDefaultToolkit().createImage(membo);
    	
    	
        final JFrame dialog = new JFrame();
        dialog.setTitle(title);
        dialog.setLayout(new BorderLayout());
        JPanel panelPrincipal = new JPanel();
        JPanel panel = new JPanel(); 
        panel.setLayout(new BorderLayout());
        JLabel label = new JLabel();
            
        label.setIcon(new ImageIcon(img));
        label.setBorder(BorderFactory.createTitledBorder(img.toString()));
        panel.add(label, BorderLayout.CENTER);
        panelPrincipal.add(panel,  BorderLayout.CENTER);
        
        panelPrincipal.setBackground(Color.LIGHT_GRAY);
        dialog.add(panelPrincipal);
        dialog.pack();
        dialog.addWindowListener(new WindowAdapter(){
            public void windowClosing(WindowEvent e) {
               dialog.dispose();
            }
        });
        
        dialog.setVisible(true);
    }
    
public static void showWindowImage(IBinaryImage imgIn, String title){
    	
    	int width = imgIn.getWidth();
    	int height = imgIn.getHeight(); 
        
        BufferedImage bi = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        WritableRaster raster = bi.getRaster();
        for(int w=0;w<width;w++){
            for(int h=0;h<height;h++){
                raster.setSample(w,h,0,imgIn.getPixel(w, h));
                raster.setSample(w,h,1,imgIn.getPixel(w, h));
                raster.setSample(w,h,2,imgIn.getPixel(w, h));
            }
        }
        ImageProducer producer = bi.getSource();
        int[] pix = new int[width * height];
        PixelGrabber grabber = new PixelGrabber(producer, 0, 0, width, height, pix, 0, width);
        try {
            if (!grabber.grabPixels())
                throw new IllegalStateException();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        MemoryImageSource membo = new MemoryImageSource(width,height,pix,0,width);
        Image img =  Toolkit.getDefaultToolkit().createImage(membo);
    	
    	
        final JFrame dialog = new JFrame();
        dialog.setTitle(title);
        dialog.setLayout(new BorderLayout());
        JPanel panelPrincipal = new JPanel();
        JPanel panel = new JPanel(); 
        panel.setLayout(new BorderLayout());
        JLabel label = new JLabel();
            
        label.setIcon(new ImageIcon(img));
        label.setBorder(BorderFactory.createTitledBorder(img.toString()));
        panel.add(label, BorderLayout.CENTER);
        panelPrincipal.add(panel,  BorderLayout.CENTER);
        
        panelPrincipal.setBackground(Color.LIGHT_GRAY);
        dialog.add(panelPrincipal);
        dialog.pack();
        dialog.addWindowListener(new WindowAdapter(){
            public void windowClosing(WindowEvent e) {
               dialog.dispose();
            }
        });
        
        dialog.setVisible(true);
    }
}