package mmtext.images.operators.uao;
import java.util.ArrayList;

import mmtext.images.GrayScaleImage;
import mmtext.images.IGrayScaleImage;
import mmtext.images.utils.ImageUtils;
import mmtext.images.utils.Pixel;

public class MaxtreeBuilder {
	private PriorityStack stack;
	private Maxtree m;
	private int nodeId; //valor do proximo node
	private IGrayScaleImage labeledImg; //matriz com o respectivo valor do node na posicao (i,j)
	private int[] labsBranch; //valor do ultimo node adicionado no nivel i no ramo atual 
	private ArrayList<Integer> level; //array com o nivel do node i
	private ArrayList<Attribute> attributes; //array com yMax e yMin do node i
	private double[] attributeValues; //array com a altura de cada node i
	private int type;
	
	public void init(IGrayScaleImage imgIn){
		this.stack = new PriorityStack();
		this.m = new Maxtree();
		m.init(imgIn.getWidth()*imgIn.getHeight());
		m.addPage();
		this.nodeId = 1;
		this.labeledImg = new GrayScaleImage(imgIn.getWidth(), imgIn.getHeight());
		for(int w=0;w<labeledImg.getWidth();w++)
			for(int h=0;h<labeledImg.getHeight();h++){
				labeledImg.setPixel(w, h, -1);
			}
		this.labsBranch = new int[256];
		for(int i=0;i<256;i++)
			labsBranch[i] = -1;
		level = new ArrayList<Integer>();
		attributes = new ArrayList<Attribute>();
	}
	
	public void printLabeledImg()
	{
		for(int h=0;h<labeledImg.getHeight();h++){
			for(int w=0;w<labeledImg.getWidth();w++)
				System.out.printf("%3d ", labeledImg.getPixel(w, h));
			System.out.println();
		}
	}
	
	/**
	 * retorna o pixel (i,j) de menor nivel de cinza
	 * 
	 * @param imageMatrix
	 * @return
	 */
	public Pixel lowestPixel(IGrayScaleImage imgIn)
	{
		int lowest = 256;
		int x = 0;
		int y = 0;
		Pixel p;
		
		for(int w=0;w < imgIn.getWidth(); w++)
			for(int h=0;h<imgIn.getHeight();h++){
				if(imgIn.getPixel(w, h) < lowest){
					lowest = imgIn.getPixel(w, h);
					x = w;
					y = h;
				}
			}
		p = new Pixel(x,y);
		return p;
	}
	
	/**
	 * conforme o algoritmo descrito no paper
	 * 
	 * @param parentValue
	 * @param value
	 */
	public void nextLabelHigher(int parentValue, int value)
	{
		Node node;
		node = new Node();
		
		level.add(value);
		node.setBrotherId(m.getNode(labsBranch[parentValue]).getSonId());
		node.setSonId(-1);
		m.setNode(node, nodeId);
		m.getNode(labsBranch[parentValue]).setSonId(nodeId);
	}
	
	/**
	 * retorna o node do ramo atual cujo nivel e' maior que value mas cujo
	 * sonId e' menor que value
	 * @param value
	 * @return
	 */
	public int levelOfParentOf(int value)
	{
		int i;
		for(i=value-1;labsBranch[i] == -1;i--);
		return i;
	}
	
	/**modificado com relacao ao algoritmo do paper, pois o mesmo parece incorreto
	 * 
	 * @param value
	 */
	public void nextLabelLower(int value)
	{
		int l;
		Node node;
		node = new Node();
		
		level.add(value);
		l = levelOfParentOf(value);
		node.setBrotherId(m.getNode(m.getNode(labsBranch[l]).getSonId()).getBrotherId());
		node.setSonId(m.getNode(labsBranch[l]).getSonId());
		m.setNode(node, nodeId);
		m.getNode(m.getNode(labsBranch[l]).getSonId()).setBrotherId(-1);
		m.getNode(labsBranch[l]).setSonId(nodeId);
		
	}
	
	/**
	 * computa o atributo de cada node da maxtree
	 */
	public void attributesCompute(){
		attributeValues = new double[attributes.size()];
		for(int i=0;i<attributes.size();i++)
			attributeValues[i] = attributes.get(i).compute();
	}
	
	/**
	 * algoritmo de flood, uma mistura do algoritmo do paper e o algoritmo escrito na
	 * referencia [9] do paper (o algoritmo do paper parece nao funcionar como deveria)
	 * 
	 * @param imageMatrix
	 * @param currentLevel
	 * @return
	 */
	public int flood(IGrayScaleImage imgIn, int currentNode)
	{
		int k, floodNode;
		Pixel p, pNext;
		int x, y;
		int currentLevel;
		int index;
		
		currentLevel = level.get(currentNode);
		
		while(!stack.getS().get(currentLevel).isEmpty()) {
			p = stack.pop();
			for(int count=0;count<8;count++){ 
				x = p.getX(); 
				y = p.getY();
				if(count == 0){
					x--;
				}
				if(count == 1){
					x++;
				}
				if(count == 2){
					y++;
				}
				if(count == 3){
					y--;
				}
				if(count == 4){
					x++;y--;
				}
				if(count == 5){
					x++;y++;
				}
				if(count == 6){
					x--;y++;
				}
				if(count == 7){
					x--;y--;
				}
				if(!ImageUtils.isIndexInImage(imgIn, x, y))
					continue;
				if(labeledImg.getPixel(x, y) == -1){//i.e. if pNext nao foi visitado
					if(imgIn.getPixel(x, y) > imgIn.getPixel(p.getX(), p.getY())){//caso A do paper
						nextLabelHigher(imgIn.getPixel(p.getX(), p.getY()), imgIn.getPixel(x,y));
						labsBranch[imgIn.getPixel(x,y)] = nodeId;
						labeledImg.setPixel(x, y, nodeId);
						Attribute a = new Attribute(type);
						a.setyMax(y);
						a.setyMin(y);
						a.setxMax(x);
						a.setxMin(x);
						a.setArea(1);
						attributes.add(a);
						nodeId++;
					}
					else if(labsBranch[imgIn.getPixel(x,y)] == -1){ //caso B
						nextLabelLower(imgIn.getPixel(x,y));
						labsBranch[imgIn.getPixel(x,y)] = nodeId;
						labeledImg.setPixel(x, y, nodeId);
						Attribute a = new Attribute(type);
						a.setyMax(y);
						a.setyMin(y);
						a.setxMax(x);
						a.setxMin(x);
						a.setArea(1);
						attributes.add(a);
						nodeId++;
					}
					else{ //caso C
						index = labsBranch[imgIn.getPixel(x,y)];
						labeledImg.setPixel(x, y, index);
						
						if(attributes.get(index).getyMax() < y)
							attributes.get(index).setyMax(y);
						if(attributes.get(index).getyMin() > y)
							attributes.get(index).setyMin(y);
						if(attributes.get(index).getxMin() > x)
							attributes.get(index).setxMin(x);
						if(attributes.get(index).getxMax() < x)
							attributes.get(index).setxMax(x);

						attributes.get(index).setArea(attributes.get(index).getArea() + 1);
					}
					pNext = new Pixel(x,y);
					stack.push(pNext,imgIn.getPixel(x,y));
					//a partir daqui o algoritmo segue como descrito em [9]
					if(imgIn.getPixel(x,y) > imgIn.getPixel(p.getX(), p.getY())){
						k = imgIn.getPixel(x,y);
						floodNode = labsBranch[k];
						while(k != currentLevel){
							k = flood(imgIn,floodNode);
							if(attributes.get(labsBranch[k]).getyMax() < attributes.get(floodNode).getyMax())
								attributes.get(labsBranch[k]).setyMax(attributes.get(floodNode).getyMax());
							if(attributes.get(labsBranch[k]).getyMin() > attributes.get(floodNode).getyMin())
								attributes.get(labsBranch[k]).setyMin(attributes.get(floodNode).getyMin());
							if(attributes.get(labsBranch[k]).getxMax() < attributes.get(floodNode).getxMax())
								attributes.get(labsBranch[k]).setxMax(attributes.get(floodNode).getxMax());
							if(attributes.get(labsBranch[k]).getxMin() > attributes.get(floodNode).getxMin())
								attributes.get(labsBranch[k]).setxMin(attributes.get(floodNode).getxMin());
							attributes.get(labsBranch[k]).setArea(attributes.get(labsBranch[k]).getArea()+attributes.get(floodNode).getArea());
							floodNode = labsBranch[k];
						}
					}
				}
			}
		}
		
		k = currentLevel - 1;
		while(k >= 0 && labsBranch[k] == -1)
			k--;
		labsBranch[currentLevel] = -1;
		return k;
	}
	
	public void buildMaxtree(IGrayScaleImage imgIn, int type){
		this.type = type;
		int currentLevel;
		Pixel p;
		
		init(imgIn);
		p = lowestPixel(imgIn);
		currentLevel = imgIn.getPixel(p.getX(), p.getY());
		
		//Inicialiazando variaveis para p = lowestPixel
		stack.push(p, currentLevel);
		labsBranch[currentLevel] = 0;
		labeledImg.setPixel(p.getX(), p.getY(), 0);
		level.add(currentLevel);
		//yValues.add(new Attribute(p.getY(), p.getY()));
		
		Attribute a = new Attribute(type);
		a.setyMax(p.getY());
		a.setyMin(p.getY());
		a.setxMin(p.getX());
		a.setxMax(p.getX());
		a.setArea(1);
		attributes.add(a);
		
		flood(imgIn,0);
		attributesCompute();
	}

	
	public void printAttributes(){
		for(int i=0;i<attributeValues.length;i++)
			System.out.print(i + ": " + attributeValues[i] + "\n");
	}
	
	public PriorityStack getStack() {
		return stack;
	}

	public void setStack(PriorityStack stack) {
		this.stack = stack;
	}

	public Maxtree getM() {
		return m;
	}

	public void setM(Maxtree m) {
		this.m = m;
	}

	public int getNodeId() {
		return nodeId;
	}

	public void setNodeId(int nodeId) {
		this.nodeId = nodeId;
	}
	
	public ArrayList<Integer> getLevel() {
		return level;
	}

	public void setLevel(ArrayList<Integer> level) {
		this.level = level;
	}

	public int[] getLabsBranch() {
		return labsBranch;
	}

	public void setLabsBranch(int[] labsBranch) {
		this.labsBranch = labsBranch;
	}

	public ArrayList<Attribute> getYValues() {
		return attributes;
	}

	public void setYValues(ArrayList<Attribute> yValues) {
		this.attributes = yValues;
	}

	public IGrayScaleImage getLabeledImg() {
		return labeledImg;
	}

	public void setLabeledImg(IGrayScaleImage labeledImg) {
		this.labeledImg = labeledImg;
	}

	public ArrayList<Attribute> getyValues() {
		return attributes;
	}

	public void setyValues(ArrayList<Attribute> yValues) {
		this.attributes = yValues;
	}

	public ArrayList<Attribute> getAttributes() {
		return attributes;
	}

	public void setAttributes(ArrayList<Attribute> attributes) {
		this.attributes = attributes;
	}

	public double[] getAttributeValues() {
		return attributeValues;
	}

	public void setAttributeValues(double[] attributeValues) {
		this.attributeValues = attributeValues;
	}

	public int getType() {
		return type;
	}

	public void setType(int type) {
		this.type = type;
	}
}