package mmtext.images.operators.uao;


import java.io.IOException;
import java.util.ArrayList;

import mmtext.images.GrayScaleImage;
import mmtext.images.IGrayScaleImage;
import mmtext.images.IRGBImage;
import mmtext.images.RGBImage;
import mmtext.images.gui.WindowImages;
import mmtext.images.utils.ImageUtils;

public class UltimateOpening {

	MaxtreeBuilder mb;
	private ArrayList<Integer> level;
	private Maxtree m;

	private double[] attributeValues;
	private int[] transformNodeLUT;
	private int[] associatedNodeLUT;
	private IGrayScaleImage transformImg;
	private IGrayScaleImage associateImg;
	private int gradualTransitions = 10;

	public UltimateOpening(){

	}

	public UltimateOpening(IGrayScaleImage img, int typeParam, double paramValue){
		MaxtreeBuilder mb = new MaxtreeBuilder();
		mb.buildMaxtree(img, typeParam);
		runUO(mb, paramValue);
	}


	public void setGradualTransitions(int x){
		gradualTransitions = x;
	}

	public void init(MaxtreeBuilder mb){
		this.mb = mb;
		m = mb.getM();
		level = mb.getLevel();
		attributeValues = mb.getAttributeValues();

		transformNodeLUT = new int[attributeValues.length];
		associatedNodeLUT = new int[attributeValues.length];
		transformImg = new GrayScaleImage(mb.getLabeledImg().getWidth(),
				mb.getLabeledImg().getHeight());
		associateImg = new GrayScaleImage(mb.getLabeledImg().getWidth(),
				mb.getLabeledImg().getHeight());
	}
	
	public void computeUO(int currentNode, int maxTr, double linkedAttributes2, double parentAttribute, int parentValue, int previousValue, double maxCriterion){
		int contrast;
		int maxContrast;
		double linkedAttributes;
		int childId;
		int pv;

		if(attributeValues[currentNode] == parentAttribute)
			contrast = level.get(currentNode) - previousValue;
		else
			contrast = level.get(currentNode) - parentValue;

		if(attributeValues[currentNode] >= maxCriterion){
			maxContrast = 0;
			linkedAttributes = -1;
		}
		else if (contrast>=maxTr) {
			maxContrast=contrast;
			linkedAttributes = attributeValues[currentNode];
		}
		else{ 
			maxContrast = maxTr; 
			linkedAttributes = linkedAttributes2;
		}
		transformNodeLUT[currentNode] = maxContrast; 
		associatedNodeLUT[currentNode] = (int)linkedAttributes+1; 
		childId = m.getNode(currentNode).getSonId();
		if (attributeValues[currentNode] == parentAttribute) 
			pv = previousValue;
		else
			pv=parentValue;
		while (childId!=-1) {
			computeUO(childId, maxContrast, linkedAttributes, attributeValues[currentNode], level.get(currentNode), pv, maxCriterion);
			childId=m.getNode(childId).getBrotherId();
		}
	}

	public void computeUO(int currentNode, int parentNode, int previous, double maxCriterion, boolean qPropag){
		int contrast;
		int maxContrast;
		double linkedAttributes;
		int childId;
		int pv = previous;
		boolean b = false;
		boolean q = false;

		if(parentNode != 0 && (attributeValues[parentNode] - attributeValues[currentNode]  <= gradualTransitions)){
			b = true;	
		}
		else
			pv = level.get(parentNode);
		
		contrast = level.get(currentNode) - pv;
		
		if(attributeValues[currentNode] > maxCriterion){
			maxContrast = 0;
			linkedAttributes = 0;
		}
		else if (transformNodeLUT[parentNode] > contrast) {
			maxContrast = transformNodeLUT[parentNode];
			linkedAttributes = associatedNodeLUT[parentNode];
		}
		else{
			maxContrast=contrast;
			if(b && qPropag){
				linkedAttributes = associatedNodeLUT[parentNode];
			}
			else{
				linkedAttributes = attributeValues[currentNode] + 1;
			}
			q = true;
		}
		transformNodeLUT[currentNode] = maxContrast;
		associatedNodeLUT[currentNode] = (int)linkedAttributes;
		childId = m.getNode(currentNode).getSonId();

		while (childId!=-1) {
			computeUO(childId, currentNode, pv, maxCriterion, q);
			childId=m.getNode(childId).getBrotherId();
		}
	}

	void generateImage(){
		for(int w=0;w<transformImg.getWidth();w++)
			for(int h=0;h<transformImg.getHeight();h++){
				transformImg.setPixel(w, h,
						transformNodeLUT[mb.getLabeledImg().getPixel(w, h)]);
				associateImg.setPixel(w, h,
						associatedNodeLUT[mb.getLabeledImg().getPixel(w, h)]);
			}
	}

	/**
	 * aplicacao do ultimate opening
	 * @param img - imagem de entrada
	 * @param typeParam - 1 altura; 2 largura; 3 area
	 * @param paramValue - valor da parametro
	 * @return
	 */
	public IGrayScaleImage runUO(IGrayScaleImage img, int typeParam,
			double paramValue){
		MaxtreeBuilder mb = new MaxtreeBuilder();
		mb.buildMaxtree(img, typeParam);
		runUO(mb, paramValue);
		mb.getM().printMaxtree(15);
		
		return getTransformImg();
	}

	public void runUO(MaxtreeBuilder mb, double d){
		int id;

		init(mb);
		transformNodeLUT[0] = 0;
		associatedNodeLUT[0] = 0;
		id = m.getNode(0).getSonId();
		while (id!=-1) {
			computeUO(id, 0, 0, d, false);
			id = m.getNode(id).getBrotherId();
		}
		generateImage();
	}

	public void printT(){
		for(int w=0;w<transformImg.getWidth();w++){
			for(int h=0;h<transformImg.getHeight();h++)
				System.out.printf("%3d ", transformImg.getPixel(w, h));
			System.out.println();
		}
	}



	// getters setters
	public MaxtreeBuilder getMb() {
		return mb;
	}

	public void setMb(MaxtreeBuilder mb) {
		this.mb = mb;
	}

	public ArrayList<Integer> getLevel() {
		return level;
	}

	public void setLevel(ArrayList<Integer> level) {
		this.level = level;
	}

	public Maxtree getM() {
		return m;
	}

	public void setM(Maxtree m) {
		this.m = m;
	}

	public int[] getTransformNodeLUT() {
		return transformNodeLUT;
	}

	public void setTransformNodeLUT(int[] transformNodeLUT) {
		this.transformNodeLUT = transformNodeLUT;
	}

	public int[] getAssociatedNodeLUT() {
		return associatedNodeLUT;
	}

	public void setAssociatedNodeLUT(int[] associatedNodeLUT) {
		this.associatedNodeLUT = associatedNodeLUT;
	}

	public IGrayScaleImage getTransformImg() {
		return transformImg;
	}

	public void setTransformImg(IGrayScaleImage transformImg) {
		this.transformImg = transformImg;
	}

	public IGrayScaleImage getAssociateImg() {
		return associateImg;
	}

	public IRGBImage getAssociateImageLabeling(){
		IRGBImage imgQcolor = new RGBImage(associateImg.getWidth(),
				associateImg.getHeight());
		for(int i=0; i < imgQcolor.getSize(); i++){
			if(associateImg.getPixel(i) != 0)
				imgQcolor.setPixel(i, ImageUtils.getColor(associateImg.getPixel(i), 1000));

		}
		return imgQcolor;
	}

	public void setAssociateImg(IGrayScaleImage associateImg) {
		this.associateImg = associateImg;
	}

	public double[] getAttributeValues() {
		return attributeValues;
	}

	public void setAttributeValues(double[] attributeValues) {
		this.attributeValues = attributeValues;
	}
}