package mmtext.images.utils;

import java.awt.Color;
import java.math.BigDecimal;

import mmtext.images.BinaryImage;
import mmtext.images.IBinaryImage;
import mmtext.images.IImage;



/**
 * Project: Computer Vision Framework
 * 
 * @author Wonder Alexandre Luz Alves
 * @advisor Ronaldo Fumio Hashimoto
 * 
 * @date 13/09/2007
 * 
 * @description
 * Esta classe define alguns metodos utilitarios para operacoes com imagens
 */
public class ImageUtils {

    /**
     * valida os indices da imagem dada
     * @param img
     * @param x
     * @param y
     * @return
     */
    public static boolean isIndexInImage(IImage img, int x, int y){
        return (x >= 0 && x < img.getWidth() && y >= 0 && y < img.getHeight());
    }
    
    public static int[] imageBounds(IBinaryImage imgIn){
    	int beginHeight = imgIn.getHeight();
        int endHeight = 0;
        int beginWidth = imgIn.getWidth();
        int endWidth = 0;
        
        for(int w=0; w < imgIn.getWidth(); w++){ //procurando os pontos que estao a imagem
            for(int h=0; h < imgIn.getHeight(); h++){
                if(imgIn.getPixel(w, h) == imgIn.getForeground() && beginHeight > h){
                    beginHeight = h;
                }
                if(imgIn.getPixel(w, h) == imgIn.getForeground() && beginWidth > w){
                    beginWidth = w;
                }
                if(imgIn.getPixel(w, h) == imgIn.getForeground() && endWidth < w){
                    endWidth = w;
                }
                if(imgIn.getPixel(w, h) == imgIn.getForeground() && endHeight < h){
                    endHeight = h;
                }                
            }
        }
        int out[] = new int[4];
        out[0] = beginWidth;
        out[1] = beginHeight;
        out[2] = endWidth;
        out[3] = endHeight;
        
        return out;
    }
 
    /**
     * Ajustar o tamanho da imagem (boundingbox)
     * @param img
     * @return
     */
    public static IBinaryImage adjustImage(IBinaryImage img){
        IBinaryImage imgOut = null;

        int beginHeight = img.getHeight();
        int endHeight = 0;
        int beginWidth = img.getWidth();
        int endWidth = 0;
        
        for(int w=0; w < img.getWidth(); w++){ //procurando os pontos que estao a imagem
            for(int h=0; h < img.getHeight(); h++){
                if(img.getPixel(w, h) == img.getForeground() && beginHeight > h){
                    beginHeight = h;
                }
                if(img.getPixel(w, h) == img.getForeground() && beginWidth > w){
                    beginWidth = w;
                }
                if(img.getPixel(w, h) == img.getForeground() && endWidth < w){
                    endWidth = w;
                }
                if(img.getPixel(w, h) == img.getForeground() && endHeight < h){
                    endHeight = h;
                }                
            }
        }
        if(endWidth - beginWidth + 1 < 0 || endHeight - beginHeight+1 < 0) return img;
        imgOut = new BinaryImage(endWidth - beginWidth + 1, endHeight - beginHeight+1); //o tamanho da imagem sao os limites + 10 //10 e uma folga
        imgOut.setBackground(img.getBackground());
        for(int w= beginWidth, i = 0; w < endWidth+1; w++, i++){ //w para imagem original e i para nova imagem 
            for(int h=beginHeight, j =0; h < endHeight+1; h++, j++){ //h para imagem original e j para nova imagem
               if(ImageUtils.isIndexInImage(imgOut, i, j))
                   imgOut.setPixel(i, j, img.getPixel(w, h));
               else
                   imgOut.setPixel(i, j, img.getBackground());
            }
        }
        
        return imgOut;
    }
    
    
    
    
    /**
     * Ajustar o tamanho da imagem (boundingbox)
     * @param img
     * @return
     */
    public static IBinaryImage adjustImageBeforeRotation(IBinaryImage img){
        IBinaryImage imgOut = new BinaryImage(img.getWidth() * 2 + img.getHeight() * 2, img.getWidth() * 2 + img.getHeight() * 2); 
        imgOut.setBackground(img.getBackground());
        imgOut.initImage(img.getBackground());
        int ww = img.getWidth() - 1;
        int hh = img.getHeight() - 1;
        for(int w=0; w < img.getWidth(); w++){ //procurando os pontos que estao a imagem
            for(int h=0; h < img.getHeight(); h++){
                imgOut.setPixel(w + ww, h + hh, img.getPixel(w, h));
            }
        }
        return imgOut;
    }
    
    public static double log2(double d) {
        return Math.log(d)/Math.log(2.0);
     }

    public static double log10(double d) {
        return Math.log(d)/Math.log(10.0);
     }
    
    public static String numberToString(double x){
        String s = new BigDecimal(x).toPlainString();
        return s;
    }

    
    /**
     * Determina a distancia entre dois pixels
     * @param w1 - coordenada da largura do primeiro pixel
     * @param h1 - coordenada da altura do primeiro pixel
     * @param w2 - coordenada da largura do segundo pixel
     * @param h2 - coordenada da altura do segundo pixel
     * @param type - tipo da distancia. 
     *               Sendo que type = 1 para euclidiana; 
     *                         type = 2 para city block; 
     *                         type = 3 para clessbord
     * @return double
     */
    public static double distance(double w1, double h1, double w2, double h2, int type){
        if(type == 1){
            return Math.sqrt(Math.pow(w1 - w2, 2) + Math.pow(h1 - h2, 2));
        }else if(type == 2){
            return Math.abs(w1 - w2) + Math.abs(h1 - h2);
        }else{
            return Math.max(Math.abs(w1 - w2), Math.abs(h1 - h2));
        }
        
    }
    
    public static Color getColor(int index, int max) {
        if (index % 2 == 1) {
            index = (index + 1) / 2;
        } else {
            index = (index + 1 + max) / 2;
        }
        return new Color((int) (0x00FFFFFF * (float) index / max));
        
    }
}
